#! /usr/local/bin/perl -wT

# HIRATA Yasuyuki's Tiny BBS (sample CGI program)
# Copyright (C) 2000, 2001, HIRATA Yasuyuki <yasu@asuka.net>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR (HIRATA YASUYUKI) ``AS IS''
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
  $RCSID = q$Id: tinybbs.cgi,v 1.2 2001/02/28 10:39:18 yasu Exp $;
#

$RCSID =~ s/\s+$//;

$MYNAME = $ENV{SCRIPT_NAME} || "";
$MYNAME =~ s[.*/][];
$MYNAME ||= "./";

require "jcode.pl";
require "./config.pl";

use FileHandle; # for OO file handle
use Fcntl qw[:flock]; # LOCK_EX

use strict;
use vars qw[%CONFIG $MYNAME $RCSID];

main();

sub main
{
    my %cgi = parse_query();
    # my %cookie = parse($ENV{HTTP_COOKIE} || "", ";"); # Not used yet

    print "Content-Type: text/html; charset=iso-2022-jp\n\n";
    print jcode::jis($CONFIG{html_header});

    $cgi{action} ||= "";
    if($cgi{action} eq "post") {
	post_article(%cgi);
	print_articles();
    } else {
	print_articles();
    }

    print jcode::jis($CONFIG{html_footer});
}

######################################################################

sub print_articles
{
    local $_;
    my $fh = new FileHandle;

    print jcode::jis($CONFIG{html_form});

    $fh->open($CONFIG{data_file}) or error($CONFIG{errmsg_file_open});
    flock $fh, LOCK_EX or error($CONFIG{errmsg_file_lock});

    while(<$fh>) {
	chomp;
	my($subject, $name, $mail, $date) = split /\t/;
	chomp(my $body = <$fh>);
	$_ ||= "" foreach ($subject, $name, $mail, $date, $body);
	$body =~ s/\t/\n/g;
	print jcode::jis($CONFIG{sub_html_article}->($body,
						     subject => $subject,
						     name => $name,
						     mail => $mail,
						     date => $date));
    }

    $fh->close;
}

sub post_article
{
    local $_;
    my %cgi = @_;
    my $fh = new FileHandle;

    $fh->open($CONFIG{data_file}) or error($CONFIG{errmsg_file_open});
    flock $fh, LOCK_EX or error($CONFIG{errmsg_file_lock});
    my @data = (<$fh>)[0..(($CONFIG{max_articles} - 1)*2 - 1)];
    $fh->close;

    foreach (qw[subject name mail]) {
	$cgi{$_} =~ tr/\r\n//d;
	$cgi{$_} =~ tr/\t/ /;
    }

    $cgi{body} =~ s/\r\n/\t/g;
    $cgi{body} =~ tr/\r\n/\t\t/;

    $cgi{body} =~ s/\t+$//;
    $cgi{body} =~ s/\t{3,}/\t\t/g;
    $cgi{body} .= "\n";

    my $time = time;

    unshift @data, ("$cgi{subject}\t$cgi{name}\t$cgi{mail}\t$time\n",
		    $cgi{body});

    $fh->open(">> $CONFIG{data_file}") or error($CONFIG{errmsg_file_open});
    flock $fh, LOCK_EX or error($CONFIG{errmsg_file_lock});
    $SIG{TERM} = $SIG{PIPE} = $SIG{KILL} = $SIG{HUP} = $SIG{USR1} = "IGNORE";

    seek $fh, 0, 0;
    truncate $fh, 0;
    print $fh @data;

    $fh->close;
    $SIG{TERM} = $SIG{PIPE} = $SIG{KILL} = $SIG{HUP} = $SIG{USR1} = "DEFAULT";
}

######################################################################

sub parse_query
{
    my $query;

    $ENV{REQUEST_METHOD} ||= ""; # for testing on a shell
    $ENV{QUERY_STRING} ||= "";   # for testing on a shell

    if($ENV{REQUEST_METHOD} eq "POST") {
	read STDIN, $query, $ENV{CONTENT_LENGTH};
    } else {
	$query = $ENV{QUERY_STRING};
    }

    (parse($query, "&"));
}

sub parse
{
    local $_;
    my $buffer = shift;
    my $delim = shift;
    my($p, %form);

    foreach (split /$delim/, $buffer) {
        my($name, $value) = split /=/;
	foreach ($name, $value) {
	    tr/ //d;
	    tr/+/ /;
	    s/%(..)/pack "C", hex $1/eg;
	}
        $form{$name} = jcode::euc($value);
    }
    (%form);
}

sub uri_escape_uric
{
    local $_ = shift;

    s/([^A-Za-z0-9\-_.!~*\'()])/sprintf "%%%02X", ord $1/eg;

    $_;
}

sub uri_escape_opaque
{
    local $_ = shift;

    s/([^A-Za-z0-9\-_.!~*\'();?:@&=+\$,])/sprintf "%%%02X", ord $1/eg;

    $_;
}

sub html_cdata
{
    local $_ = shift;

    s/&/&amp;/g;
    s/</&lt;/g;
    s/>/&gt;/g;
    s/\"/&quot;/g;

    $_;
}

sub error
{
    $CONFIG{sub_html_error}->(@_);
    print jcode::jis($CONFIG{html_footer});
    exit;
}

__END__
